@@ -1,26 +1,32 @@
 /**
-* Copyright (C) Mellanox Technologies Ltd. 2001-2012.  ALL RIGHTS RESERVED.
+* Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2001-2012. ALL RIGHTS RESERVED.
 *
 * See file LICENSE for terms.
 */
 
 #include "test_helpers.h"
 
+#include <ucs/async/async.h>
 #include <ucs/sys/math.h>
 #include <ucs/sys/sys.h>
 #include <ucs/sys/string.h>
+#include <ucs/config/global_opts.h>
 #include <ucs/config/parser.h>
 
-#include <sys/resource.h>
 #include <set>
 
+extern "C" {
+// On some platforms users have to declare environ explicitly
+extern char** environ;
+}
+
 namespace ucs {
 
 typedef std::pair<std::string, ::testing::TimeInMillis> test_result_t;
 
-const double test_timeout_in_sec = 60.;
+const double test_timeout_in_sec = 180.;
 
-const double watchdog_timeout_default = 900.; // 15 minutes
+double watchdog_timeout = 900.; // 15 minutes
 
 static test_watchdog_t watchdog;
 
@@ -72,7 +78,7 @@ void *watchdog_func(void *arg)
             watchdog.state = WATCHDOG_DEFAULT_SET;
             break;
         case WATCHDOG_DEFAULT_SET:
-            watchdog.timeout     = watchdog_timeout_default;
+            watchdog.timeout     = watchdog_timeout;
             watchdog.state       = WATCHDOG_RUN;
             watchdog.kill_signal = SIGABRT;
             break;
@@ -110,7 +116,7 @@ void watchdog_set(test_watchdog_state_t new_state, double new_timeout)
 
 void watchdog_set(test_watchdog_state_t new_state)
 {
-    watchdog_set(new_state, watchdog_timeout_default);
+    watchdog_set(new_state, watchdog_timeout);
 }
 
 void watchdog_set(double new_timeout)
@@ -137,6 +143,7 @@ WATCHDOG_DEFINE_GETTER(kill_signal, int)
 int watchdog_start()
 {
     pthread_mutexattr_t mutex_attr;
+    ucs_status_t status;
     int ret;
 
     ret = pthread_mutexattr_init(&mutex_attr);
@@ -167,13 +174,14 @@ int watchdog_start()
 
     pthread_mutex_lock(&watchdog.mutex);
     watchdog.state          = WATCHDOG_RUN;
-    watchdog.timeout        = watchdog_timeout_default;
+    watchdog.timeout        = watchdog_timeout;
     watchdog.kill_signal    = SIGABRT;
     watchdog.watched_thread = pthread_self();
     pthread_mutex_unlock(&watchdog.mutex);
 
-    ret = pthread_create(&watchdog.thread, NULL, watchdog_func, NULL);
-    if (ret != 0) {
+    status = ucs_pthread_create(&watchdog.thread, watchdog_func, NULL,
+                                "watchdog");
+    if (status != UCS_OK) {
         goto err_destroy_barrier;
     }
 
@@ -316,35 +324,37 @@ void analyze_test_results()
 int test_time_multiplier()
 {
     int factor = 1;
-#if _BullseyeCoverage
-    factor *= 10;
-#endif
     if (RUNNING_ON_VALGRIND) {
         factor *= 20;
     }
+#if _BullseyeCoverage
+    factor *= 10;
+#endif
+#ifdef __SANITIZE_ADDRESS__
+    factor *= 20;
+#endif
+
     return factor;
 }
 
 ucs_time_t get_deadline(double timeout_in_sec)
 {
-    return ucs_get_time() + ucs_time_from_sec(timeout_in_sec *
-                                              test_time_multiplier());
+    return ucs_get_time() +
+           ucs_time_from_sec(ucs_min(watchdog_get_timeout() * 0.75,
+                                     timeout_in_sec * test_time_multiplier()));
 }
 
 int max_tcp_connections()
 {
     static int max_conn = 0;
 
-    if (!max_conn) {
-        max_conn = 65535 - 1024; /* limit on number of ports */
-
-        /* Limit numer of endpoints to number of open files, for TCP */
-        struct rlimit rlim;
-        int ret = getrlimit(RLIMIT_NOFILE, &rlim);
-        if (ret == 0) {
-            /* assume no more than 100 fd-s are already used */
-            max_conn = ucs_min((static_cast<int>(rlim.rlim_cur) - 100) / 2, max_conn);
-        }
+    if (max_conn == 0) {
+        /* assume no more than 100 fd-s are already used and consider
+         * that each side of the connection could create 2 socket fds
+         * (1 - from ucp_ep_create() API function, 2 - from accepting
+         * the remote connection), i.e. 4 socket fds per connection  */
+        max_conn = std::min((ucs_sys_max_open_files() - 100) / 4,
+                            65535 - 1024/* limit on number of ports */);
     }
 
     return max_conn;
@@ -429,11 +439,63 @@ void safe_usleep(double usec) {
 }
 
 bool is_inet_addr(const struct sockaddr* ifa_addr) {
-    return (ifa_addr->sa_family == AF_INET) ||
-           (ifa_addr->sa_family == AF_INET6);
+    if (ifa_addr == NULL) {
+        return false;
+    }
+
+    if (ifa_addr->sa_family == AF_INET6) {
+        /* Skip IPv6 link-local and loopback address, that could not be used for
+           connection establishment */
+        auto saddr6 = (const struct sockaddr_in6*)ifa_addr;
+        return !IN6_IS_ADDR_LOOPBACK(&saddr6->sin6_addr) &&
+               !IN6_IS_ADDR_LINKLOCAL(&saddr6->sin6_addr);
+    } else {
+        return ifa_addr->sa_family == AF_INET;
+    }
 }
 
-static std::vector<std::string> read_dir(const std::string& path)
+static bool netif_has_sysfs_file(const char *ifa_name, const char *file_name)
+{
+    std::string path(PATH_MAX, '\0');
+
+    ucs_snprintf_safe(&path[0], path.size(), "/sys/class/net/%s/%s", ifa_name,
+                      file_name);
+
+    struct stat st;
+    return stat(path.c_str(), &st) >= 0;
+}
+
+bool is_interface_usable(struct ifaddrs *ifa)
+{
+    return ucs_netif_flags_is_active(ifa->ifa_flags) &&
+           ucs::is_inet_addr(ifa->ifa_addr) &&
+           !netif_has_sysfs_file(ifa->ifa_name, "bridge") &&
+           !netif_has_sysfs_file(ifa->ifa_name, "brport") &&
+           !netif_has_sysfs_file(ifa->ifa_name, "wireless");
+}
+
+
+ssize_t get_proc_self_status_field(const std::string &parameter)
+{
+    const std::string path("/proc/self/status");
+    std::ifstream proc_stats(path);
+    std::string line, name;
+    ssize_t value;
+
+    while (std::getline(proc_stats, line)) {
+        if (!(std::istringstream(line) >> name >> value)) {
+            continue;
+        }
+        if (name == (parameter + ":")) {
+            return value;
+        }
+    }
+
+    UCS_TEST_MESSAGE << path << " does not contain " << parameter << " value";
+    return -1;
+}
+
+std::vector<std::string> read_dir(const std::string &path)
 {
     std::vector<std::string> result;
     struct dirent *entry;
@@ -455,29 +517,45 @@ out_close:
     return result;
 }
 
-static std::set<std::string> get_all_rdmacm_net_devices()
+static std::map<std::string, std::string> get_all_rdmacm_net_devices()
 {
     static const std::string sysfs_ib_dir  = "/sys/class/infiniband";
     static const std::string sysfs_net_dir = "/sys/class/net";
     static const std::string ndevs_fmt     = sysfs_ib_dir +
                                              "/%s/ports/%d/gid_attrs/ndevs/0";
     static const std::string node_guid_fmt = sysfs_ib_dir + "/%s/node_guid";
-    std::set<std::string> devices;
+    std::map<std::string, std::string> devices;
     char dev_name[32];
     char guid_buf[32];
     ssize_t nread;
     int port_num;
 
+    if (ucs::is_aws()) {
+        return devices;
+    }
+
     std::vector<std::string> ndevs = read_dir(sysfs_net_dir);
 
     /* Enumerate IPoIB and RoCE devices which have direct mapping to an RDMA
      * device.
      */
     for (size_t i = 0; i < ndevs.size(); ++i) {
-        std::string infiniband_dir = sysfs_net_dir + "/" + ndevs[i] +
-                                     "/device/infiniband";
-        if (!read_dir(infiniband_dir).empty()) {
-            devices.insert(ndevs[i]);
+        std::string infiniband_dir          = sysfs_net_dir + "/" + ndevs[i] +
+                                              "/device/infiniband";
+        std::vector<std::string> ib_devices = read_dir(infiniband_dir);
+
+        if (!ib_devices.empty()) {
+            std::string ib_device = ib_devices.front();
+            if (ib_device.rfind("smi", 0) == 0) {
+                /* Skip SMI device */
+                continue;
+            }
+
+            std::string ports_dir = infiniband_dir + "/" + ib_device +
+                                    "/ports";
+            std::string ib_port   = read_dir(ports_dir).front();
+
+            devices.emplace(ndevs[i], ib_device + ":" + ib_port);
         }
     }
 
@@ -504,7 +582,9 @@ static std::set<std::string> get_all_rdmacm_net_devices()
 
             /* use the device if node_guid != 0 */
             if (strstr(guid_buf, "0000:0000:0000:0000") == NULL) {
-                devices.insert(ucs_strtrim(dev_name));
+                devices.emplace(ucs_strtrim(dev_name),
+                                std::string(ndev_name) + ":" +
+                                ucs::to_string(port_num));
             }
         }
     }
@@ -512,16 +592,36 @@ static std::set<std::string> get_all_rdmacm_net_devices()
     return devices;
 }
 
-bool is_rdmacm_netdev(const char *ifa_name) {
+std::string get_rdmacm_netdev(const char *ifa_name)
+{
     static bool initialized = false;
-    static std::set<std::string> devices;
+    static std::map<std::string, std::string> devices;
 
     if (!initialized) {
         devices     = get_all_rdmacm_net_devices();
         initialized = true;
     }
 
-    return devices.find(ifa_name) != devices.end();
+    auto dev = devices.find(ifa_name);
+    return (dev != devices.end()) ? dev->second : "";
+}
+
+bool is_rdmacm_netdev(const char *ifa_name)
+{
+    return !get_rdmacm_netdev(ifa_name).empty();
+}
+
+bool is_aws()
+{
+    static bool result, initialized = false;
+
+    if (!initialized) {
+        const char *str = getenv("CLOUD_TYPE");
+        result          = (str != NULL) && !strcmp(str, "aws");
+        initialized     = true;
+    }
+
+    return result;
 }
 
 uint16_t get_port() {
@@ -531,7 +631,7 @@ uint16_t get_port() {
     socklen_t len = sizeof(ret_addr);
     uint16_t port;
 
-    status = ucs_socket_create(AF_INET, SOCK_STREAM, &sock_fd);
+    status = ucs_socket_create(AF_INET, SOCK_STREAM, 0, &sock_fd);
     EXPECT_EQ(status, UCS_OK);
 
     memset(&addr_in, 0, sizeof(struct sockaddr_in));
@@ -555,8 +655,18 @@ uint16_t get_port() {
     return port;
 }
 
-void *mmap_fixed_address() {
-    return (void*)0xff0000000;
+mmap_fixed_address::mmap_fixed_address(size_t length) : m_length(length) {
+    m_ptr = mmap(NULL, m_length, PROT_READ | PROT_WRITE,
+                 MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+    if (m_ptr == MAP_FAILED) {
+        UCS_TEST_ABORT("mmap failed to allocate memory region");
+    }
+}
+
+mmap_fixed_address::~mmap_fixed_address() {
+    if (m_ptr != NULL) {
+        munmap(m_ptr, m_length);
+    }
 }
 
 std::string compact_string(const std::string &str, size_t length)
@@ -568,27 +678,63 @@ std::string compact_string(const std::string &str, size_t length)
     return str.substr(0, length) + "..." + str.substr(str.length() - length);
 }
 
-sock_addr_storage::sock_addr_storage() : m_size(0), m_is_valid(false) {
+std::string exit_status_info(int exit_status)
+{
+    std::stringstream ss;
+
+    if (WIFEXITED(exit_status)) {
+        ss << ", exited with status " << WEXITSTATUS(exit_status);
+    }
+    if (WIFSIGNALED(exit_status)) {
+        ss << ", signaled with status " << WTERMSIG(exit_status);
+    }
+    if (WIFSTOPPED(exit_status)) {
+        ss << ", stopped with status " << WSTOPSIG(exit_status);
+    }
+
+    return ss.str().substr(2, std::string::npos);
+}
+
+size_t limit_buffer_size(size_t size)
+{
+    return std::min(size, std::min(ucs_get_phys_mem_size() / 16,
+                                   ucs_get_memfree_size() / 4));
+}
+
+sock_addr_storage::sock_addr_storage() :
+        m_size(0), m_is_valid(false), m_is_rdmacm_netdev(false)
+{
     memset(&m_storage, 0, sizeof(m_storage));
 }
 
-sock_addr_storage::sock_addr_storage(const ucs_sock_addr_t &ucs_sock_addr) {
+sock_addr_storage::sock_addr_storage(const ucs_sock_addr_t &ucs_sock_addr,
+                                     bool is_rdmacm_netdev,
+                                     std::string netdev_name,
+                                     std::string rdmacm_netdev_name)
+{
     if (sizeof(m_storage) < ucs_sock_addr.addrlen) {
         memset(&m_storage, 0, sizeof(m_storage));
-        m_size     = 0;
-        m_is_valid = false;
+        m_size             = 0;
+        m_is_valid         = false;
+        m_is_rdmacm_netdev = false;
+        m_netdev_name      = "";
     } else {
-        set_sock_addr(*ucs_sock_addr.addr, ucs_sock_addr.addrlen);
+        set_sock_addr(*ucs_sock_addr.addr, ucs_sock_addr.addrlen,
+                      is_rdmacm_netdev, netdev_name);
     }
 }
 
 void sock_addr_storage::set_sock_addr(const struct sockaddr &addr,
-                                      const size_t size) {
+                                      const size_t size, bool is_rdmacm_netdev,
+                                      std::string netdev_name)
+{
     ASSERT_GE(sizeof(m_storage), size);
     ASSERT_TRUE(ucs::is_inet_addr(&addr));
     memcpy(&m_storage, &addr, size);
-    m_size     = size;
-    m_is_valid = true;
+    m_size               = size;
+    m_is_valid           = true;
+    m_is_rdmacm_netdev   = is_rdmacm_netdev;
+    m_netdev_name        = netdev_name;
 }
 
 void sock_addr_storage::reset_to_any() {
@@ -646,6 +792,16 @@ uint16_t sock_addr_storage::get_port() const {
     }
 }
 
+bool sock_addr_storage::is_rdmacm_netdev() const
+{
+    return m_is_rdmacm_netdev;
+}
+
+std::string sock_addr_storage::netdev_name() const
+{
+    return m_netdev_name;
+}
+
 size_t sock_addr_storage::get_addr_size() const {
     return m_size;
 }
@@ -663,30 +819,54 @@ std::string sock_addr_storage::to_str() const {
     return ucs_sockaddr_str(get_sock_addr_ptr(), str, sizeof(str));
 }
 
+std::string sock_addr_storage::to_ip_str() const {
+    char str[UCS_SOCKADDR_STRING_LEN];
+    ucs_status_t status;
+
+    status = ucs_sockaddr_get_ipstr(get_sock_addr_ptr(), str, sizeof(str));
+    ASSERT_UCS_OK(status);
+    return str;
+}
+
 const struct sockaddr* sock_addr_storage::get_sock_addr_ptr() const {
     return m_is_valid ? (struct sockaddr *)(&m_storage) : NULL;
 }
 
+const void* sock_addr_storage::get_sock_addr_in_buf() const {
+    const struct sockaddr* saddr = get_sock_addr_ptr();
+
+    ucs_assert_always(saddr != NULL);
+    ucs_assert_always((saddr->sa_family == AF_INET) ||
+                      (saddr->sa_family == AF_INET6));
+
+    return (saddr->sa_family == AF_INET) ?
+           (const void*)&((struct sockaddr_in*)saddr)->sin_addr :
+           (const void*)&((struct sockaddr_in6*)saddr)->sin6_addr;
+}
+
 std::ostream& operator<<(std::ostream& os, const sock_addr_storage& sa_storage)
 {
     return os << ucs::sockaddr_to_str(sa_storage.get_sock_addr_ptr());
 }
 
-auto_buffer::auto_buffer(size_t size) : m_ptr(malloc(size)) {
-    if (!m_ptr) {
-        UCS_TEST_ABORT("Failed to allocate memory");
-    }
+auto_buffer::auto_buffer(size_t size) : m_buf(size) {
 }
 
-auto_buffer::~auto_buffer()
-{
-    free(m_ptr);
-}
-
-void* auto_buffer::operator*() const {
-    return m_ptr;
+void* auto_buffer::operator*() {
+    return as<void>();
 };
 
+scoped_log_level::scoped_log_level(ucs_log_level_t level)
+    : m_prev_level(ucs_global_opts.log_component.log_level)
+{
+    ucs_global_opts.log_component.log_level = level;
+}
+
+scoped_log_level::~scoped_log_level()
+{
+    ucs_global_opts.log_component.log_level = m_prev_level;
+}
+
 namespace detail {
 
 message_stream::message_stream(const std::string& title) {
@@ -705,7 +885,29 @@ message_stream::~message_stream() {
 
 } // detail
 
-std::vector<std::vector<ucs_memory_type_t> > supported_mem_type_pairs() {
+scoped_async_lock::scoped_async_lock(ucs_async_context_t &async) :
+    m_async(async)
+{
+    UCS_ASYNC_BLOCK(&m_async);
+}
+
+scoped_async_lock::~scoped_async_lock()
+{
+    UCS_ASYNC_UNBLOCK(&m_async);
+}
+
+scoped_mutex_lock::scoped_mutex_lock(pthread_mutex_t &mutex) : m_mutex(mutex)
+{
+    pthread_mutex_lock(&m_mutex);
+}
+
+scoped_mutex_lock::~scoped_mutex_lock()
+{
+    pthread_mutex_unlock(&m_mutex);
+}
+
+const std::vector<std::vector<ucs_memory_type_t> >& supported_mem_type_pairs()
+{
     static std::vector<std::vector<ucs_memory_type_t> > result;
 
     if (result.empty()) {
