@@ -146,6 +146,13 @@ extern MPI_Datatype MPIR_Datatype_index_to_predefined[MPIR_DATATYPE_N_PREDEFINED
 
 void MPIR_Datatype_free(MPIR_Datatype * ptr);
 void MPIR_Datatype_get_flattened(MPI_Datatype type, void **flattened, int *flattened_sz);
+int MPIR_Datatype_commit_pairtypes(void);
+int MPIR_Datatype_init_predefined(void);
+int MPIR_Datatype_builtintype_alignment(MPI_Datatype type);
+
+/* internal debugging functions */
+void MPII_Datatype_printf(MPI_Datatype type, int depth, MPI_Aint displacement, int blocklength,
+                          int header);
 
 #define MPIR_Datatype_ptr_add_ref(datatype_ptr) MPIR_Object_add_ref((datatype_ptr))
 
@@ -380,7 +387,7 @@ static inline int MPIR_Datatype_set_contents(MPIR_Datatype * new_dtp,
                                              const MPI_Aint array_of_counts[],
                                              const MPI_Datatype array_of_types[])
 {
-    int mpi_errno;
+    int mpi_errno = MPI_SUCCESS;
     MPI_Aint struct_sz, ints_sz, aints_sz, counts_sz, types_sz, contents_size;
     MPIR_Datatype_contents *cp;
     MPIR_Datatype *old_dtp;
@@ -465,11 +472,12 @@ static inline int MPIR_Datatype_set_contents(MPIR_Datatype * new_dtp,
     for (MPI_Aint i = 0; i < nr_types; i++) {
         if (!HANDLE_IS_BUILTIN(array_of_types[i])) {
             MPIR_Datatype_get_ptr(array_of_types[i], old_dtp);
+            MPIR_Datatype_valid_ptr(old_dtp, mpi_errno);
             MPIR_Datatype_ptr_add_ref(old_dtp);
         }
     }
 
-    return MPI_SUCCESS;
+    return mpi_errno;
 }
 
 MPL_STATIC_INLINE_PREFIX void MPIR_Datatype_access_contents(MPIR_Datatype_contents * cp,
