@@ -19,24 +19,23 @@
  */
 
 int MPIR_Allgatherv_intra_recursive_doubling(const void *sendbuf,
-                                             int sendcount,
+                                             MPI_Aint sendcount,
                                              MPI_Datatype sendtype,
                                              void *recvbuf,
-                                             const int *recvcounts,
-                                             const int *displs,
+                                             const MPI_Aint * recvcounts,
+                                             const MPI_Aint * displs,
                                              MPI_Datatype recvtype,
-                                             MPIR_Comm * comm_ptr, MPIR_Errflag_t * errflag)
+                                             MPIR_Comm * comm_ptr, MPIR_Errflag_t errflag)
 {
     int comm_size, rank, j, i;
     int mpi_errno = MPI_SUCCESS;
-    int mpi_errno_ret = MPI_SUCCESS;
     MPI_Status status;
     MPI_Aint recvtype_extent, recvtype_sz;
-    MPI_Aint curr_cnt, last_recv_cnt;
-    int dst, total_count;
+    MPI_Aint curr_cnt, last_recv_cnt, total_count;
+    int dst;
     void *tmp_buf;
-    int mask, dst_tree_root, my_tree_root, position,
-        send_offset, recv_offset, nprocs_completed, k, offset, tmp_mask, tree_root;
+    int mask, dst_tree_root, my_tree_root, nprocs_completed, k, tmp_mask, tree_root;
+    MPI_Aint position, send_offset, recv_offset, offset;
     MPIR_CHKLMEM_DECL(1);
 
     comm_size = comm_ptr->local_size;
@@ -114,18 +113,14 @@ int MPIR_Allgatherv_intra_recursive_doubling(const void *sendbuf,
                                       ((char *) tmp_buf + recv_offset * recvtype_sz),
                                       (total_count - recv_offset) * recvtype_sz, MPI_BYTE, dst,
                                       MPIR_ALLGATHERV_TAG, comm_ptr, &status, errflag);
+            MPIR_ERR_CHECK(mpi_errno);
             if (mpi_errno) {
-                /* for communication errors, just record the error but continue */
-                *errflag =
-                    MPIX_ERR_PROC_FAILED ==
-                    MPIR_ERR_GET_CLASS(mpi_errno) ? MPIR_ERR_PROC_FAILED : MPIR_ERR_OTHER;
-                MPIR_ERR_SET(mpi_errno, *errflag, "**fail");
-                MPIR_ERR_ADD(mpi_errno_ret, mpi_errno);
                 last_recv_cnt = 0;
-            } else
+            } else {
                 /* for convenience, recv is posted for a bigger amount
                  * than will be sent */
                 MPIR_Get_count_impl(&status, recvtype, &last_recv_cnt);
+            }
             curr_cnt += last_recv_cnt;
         }
 
@@ -181,14 +176,7 @@ int MPIR_Allgatherv_intra_recursive_doubling(const void *sendbuf,
                     mpi_errno = MPIC_Send(((char *) tmp_buf + offset * recvtype_sz),
                                           last_recv_cnt * recvtype_sz,
                                           MPI_BYTE, dst, MPIR_ALLGATHERV_TAG, comm_ptr, errflag);
-                    if (mpi_errno) {
-                        /* for communication errors, just record the error but continue */
-                        *errflag =
-                            MPIX_ERR_PROC_FAILED ==
-                            MPIR_ERR_GET_CLASS(mpi_errno) ? MPIR_ERR_PROC_FAILED : MPIR_ERR_OTHER;
-                        MPIR_ERR_SET(mpi_errno, *errflag, "**fail");
-                        MPIR_ERR_ADD(mpi_errno_ret, mpi_errno);
-                    }
+                    MPIR_ERR_CHECK(mpi_errno);
                     /* last_recv_cnt was set in the previous
                      * receive. that's the amount of data to be
                      * sent now. */
@@ -205,14 +193,9 @@ int MPIR_Allgatherv_intra_recursive_doubling(const void *sendbuf,
 
                     mpi_errno = MPIC_Recv(((char *) tmp_buf + offset * recvtype_sz),
                                           (total_count - offset) * recvtype_sz, MPI_BYTE,
-                                          dst, MPIR_ALLGATHERV_TAG, comm_ptr, &status, errflag);
+                                          dst, MPIR_ALLGATHERV_TAG, comm_ptr, &status);
+                    MPIR_ERR_CHECK(mpi_errno);
                     if (mpi_errno) {
-                        /* for communication errors, just record the error but continue */
-                        *errflag =
-                            MPIX_ERR_PROC_FAILED ==
-                            MPIR_ERR_GET_CLASS(mpi_errno) ? MPIR_ERR_PROC_FAILED : MPIR_ERR_OTHER;
-                        MPIR_ERR_SET(mpi_errno, *errflag, "**fail");
-                        MPIR_ERR_ADD(mpi_errno_ret, mpi_errno);
                         last_recv_cnt = 0;
                     } else
                         /* for convenience, recv is posted for a
@@ -247,11 +230,6 @@ int MPIR_Allgatherv_intra_recursive_doubling(const void *sendbuf,
 
   fn_exit:
     MPIR_CHKLMEM_FREEALL();
-    if (mpi_errno_ret)
-        mpi_errno = mpi_errno_ret;
-    else if (*errflag != MPIR_ERR_NONE)
-        MPIR_ERR_SET(mpi_errno, *errflag, "**coll_fail");
-
     return mpi_errno;
   fn_fail:
     goto fn_exit;
