@@ -1,9 +1,9 @@
-! This file created from test/mpi/errors/f77/io/uerrhandf.f with f77tof90
-! -*- Mode: Fortran; -*- 
+! This file created from errors/f77/io/uerrhandf.f with f77tof90
 !
-!  (C) 2013 by Argonne National Laboratory.
-!      See COPYRIGHT in top-level directory.
+! Copyright (C) by Argonne National Laboratory
+!     See COPYRIGHT in top-level directory
 !
+
       program main
       use mpi
       integer (kind=MPI_ADDRESS_KIND) asize
@@ -49,14 +49,10 @@
       call mpi_comm_set_errhandler( wdup, comm_errh, ierr )
       call mpi_comm_size( wdup, wdsize, ierr )
       call mpi_send( id, 1, MPI_INTEGER, wdsize, -37, wdup, ierr )
-      if (ierr .eq. MPI_SUCCESS) then
-         print *, ' Failed to detect error in use of MPI_SEND'
+! NOTE: ierr may be MPI_SUCCESS but handler should be invoked
+      if (iseen(1) .ne. 1) then
          errs = errs + 1
-      else
-         if (iseen(1) .ne. 1) then
-            errs = errs + 1
-            print *, ' Failed to increment comm error counter'
-         endif
+         print *, ' Failed to increment comm error counter'
       endif
 
       asize = 2*sizeofint
@@ -70,14 +66,10 @@
       asize = 0
       call mpi_put( winbuf, 1, MPI_INT, wdsize, asize, 1, MPI_INT, winh, &
       &     ierr )
-      if (ierr .eq. MPI_SUCCESS) then
-         print *, ' Failed to detect error in use of MPI_PUT'
+! NOTE: ierr may be MPI_SUCCESS but handler should be invoked
+      if (iseen(3) .ne. 1) then
          errs = errs + 1
-      else
-         if (iseen(3) .ne. 1) then
-            errs = errs + 1
-            print *, ' Failed to increment win error counter'
-         endif
+         print *, ' Failed to increment win error counter'
       endif
 
       call mpi_file_open( MPI_COMM_SELF, 'ftest', MPI_MODE_CREATE + &
@@ -91,14 +83,10 @@
       offset = -100
       call mpi_file_read_at( fh, offset, winbuf, 1, MPI_INTEGER, status, &
       &     ierr )
-      if (ierr .eq. MPI_SUCCESS) then
-         print *, ' Failed to detect error in use of MPI_PUT'
+! NOTE: ierr may be MPI_SUCCESS but handler should be invoked
+      if (iseen(2) .ne. 1) then
          errs = errs + 1
-      else
-         if (iseen(2) .ne. 1) then
-            errs = errs + 1
-            print *, ' Failed to increment file error counter'
-         endif
+         print *, ' Failed to increment file error counter'
       endif
 
       call mpi_comm_free( wdup, ierr )
@@ -110,7 +98,6 @@
       call mpi_errhandler_free( file_errh, ierr )
       
       call mtest_finalize( errs )
-      call mpi_finalize( ierr )
       end
 !
       subroutine comm_errh_fn( comm, ec )
